@@ -1,24 +1,15 @@
-from __future__ import absolute_import
-from builtins import str, bytes
 import os
 import time
-import sqlite3
 
-from werkzeug.local import LocalProxy
-from flask import Flask, g, current_app, Blueprint, Markup
+from flask import Flask, Blueprint, Markup
 from flask.helpers import send_from_directory
 from flaskext.markdown import Markdown
 from jinja2 import FileSystemLoader
-from .cache import cache
-from sqlalchemy import create_engine
-from sqlalchemy.engine import Engine
-from sqlalchemy import event
 from babel import dates
+from humanize import naturaltime
 
-from . import shortcodes
-
-# from chill.resource import resource
-# from chill.page import page
+from chill import shortcodes
+from chill import database
 
 
 class ChillFlask(Flask):
@@ -50,46 +41,6 @@ class ChillFlask(Flask):
         )
 
 
-def connect_to_database():
-    """
-    Return the engine. Echo all sql statements if in DEBUG mode.
-    """
-    def sqlite_readonly_connect():
-
-        db_file = current_app.config.get("CHILL_DATABASE_URI")[len('sqlite:///'):]
-        if db_file and not db_file.startswith(':'):
-            # Open the database connection in read only mode
-            return sqlite3.connect(
-                "file:{}?mode=ro".format(db_file), uri=True
-            )
-        else:
-            return sqlite3.connect(current_app.config.get("CHILL_DATABASE_URI"))
-
-    if current_app.config.get("database_readonly") and current_app.config.get("is_sqlite_database"):
-        return create_engine(current_app.config["CHILL_DATABASE_URI"], creator=sqlite_readonly_connect)
-    else:
-        return create_engine(current_app.config["CHILL_DATABASE_URI"])
-
-
-def get_db():
-    db = getattr(g, "_database", None)
-    if db is None:
-        db = g._database = connect_to_database()
-        if str(db.url).startswith("sqlite://"):
-            # Enable foreign key support so 'on update' and 'on delete' actions
-            # will apply. This needs to be set for each db connection.
-            @event.listens_for(Engine, "connect")
-            def set_sqlite_pragma(dbapi_connection, connection_record):
-                cursor = dbapi_connection.cursor()
-                cursor.execute("PRAGMA foreign_keys=ON")
-                cursor.close()
-
-    return db
-
-
-db = LocalProxy(get_db)
-
-
 def multiple_directory_files_loader(*args):
     """
     Loads all the files in each directory as values in a dict with the key
@@ -124,10 +75,6 @@ def make_app(config=None, database_readonly=False, **kw):
         )
         app.config.from_pyfile(config_file)
     app.config.update(kw, database_readonly=database_readonly)
-    is_sqlite_database = str(app.config.get("CHILL_DATABASE_URI")).startswith("sqlite://")
-    app.config["is_sqlite_database"] = is_sqlite_database
-
-    cache.init_app(app)
 
     # Set the freezer destination path to be absolute if needed.
     freeze_folder = app.config.get("FREEZER_DESTINATION", None)
@@ -189,11 +136,8 @@ def make_app(config=None, database_readonly=False, **kw):
     # Set the jinja2 template folder eith fallback for app.template_folder
     app.jinja_env.loader = FileSystemLoader(app.config.get("THEME_TEMPLATE_FOLDER"))
 
-    @app.teardown_appcontext
-    def teardown_db(exception):
-        db = getattr(g, "_database", None)
-        if db is not None:
-            db.dispose()
+    app.logger.debug("Database init_app")
+    database.init_app(app)
 
     # STATIC_URL='http://cdn.example.com/whatever/works/'
     @app.context_processor
@@ -234,11 +178,12 @@ def make_app(config=None, database_readonly=False, **kw):
 
     @app.template_filter("timedelta")
     def timedelta(value):
-        "time delta"
-        return dates.format_timedelta(value)
+        "time delta using humanize.time.naturaltime()"
+        return naturaltime(value)
 
     # Add the markdown filter for the templates
-    md = Markdown(app)
+    # https://pythonhosted.org/Flask-Markdown/
+    Markdown(app)
 
     @app.template_filter("readfile")
     def readfile(filename):
@@ -265,9 +210,14 @@ def make_app(config=None, database_readonly=False, **kw):
                 # py2 and py3 compat
                 content = bytes(f.read(), "utf-8").decode("utf-8")
             return content
+        else:
+            app.logger.warning(
+                "The DOCUMENT_FOLDER setting in site.cfg is not set to a value. Can't use 'readfile' filter."
+            )
+            return filename
 
         app.logger.warn(
-            "jinja2 filter 'readfile' can't find file: '{0}'".format(filename)
+            f"jinja2 filter 'readfile' can't find file: '{filename}' at DOCUMENT_FOLDER path: {document_folder}"
         )
         return filename
 
